import { Observable } from 'rxjs';
import { Store } from '../index';
export declare type UndoOrRedo = 'redo' | 'undo';
export declare abstract class UndoManager<StateType, UndoStateType> {
    protected readonly store: Store<StateType>;
    protected maxDepth: number;
    private stack;
    private currentStateIndex;
    private canUndoSubject;
    private canRedoSubject;
    private stateSubject;
    private currentCollectKey?;
    private collectDebouncer;
    /**
     * An observable that emits the result of `canUndo()` every time that value changes.
     */
    canUndo$: Observable<boolean>;
    /**
     * An observable that emits the result of `canRedo()` every time that value changes.
     */
    canRedo$: Observable<boolean>;
    /**
     * An observable that emits the current state every time it changes.
     */
    state$: Observable<UndoStateType>;
    /**
     * @param maxDepth The maximum size of the history before discarding the oldest state. `0` means no limit.
     */
    constructor(store: Store<StateType>, maxDepth?: number);
    /**
     * Discard all history and push the current state.
     */
    reset(): void;
    /**
     * Add the current state to the undo history. Any states that could be reached using `redo()` are discarded.
     *
     * @param collectKey When specified, multiple pushes in a row with the same key will be collected into a single undo state. This is useful e.g. for collecting changes from a text input into larger undo states, rather than undoing one character at a time. After a push with a different key or another undo change (like `.undo()` or `.reset()`), collecting will stop and the next push will be in a new undo state.
     * @param collectDebounce If at least this many milliseconds elapse with no other push, the next one will be to a new undo state regardless of its `collectKey`. Defaults to `undefined`, which sets no such timeout.
     */
    pushCurrentState({ collectKey, collectDebounce, }?: {
        collectKey?: string | undefined;
        collectDebounce?: number | undefined;
    }): void;
    /**
     * @returns whether any states are available for `undo()`
     */
    canUndo(): boolean;
    /**
     * @returns whether any states are available for `redo()`
     */
    canRedo(): boolean;
    /**
     * Move backward one step in the history of states saved via `pushCurrentState()`, setting the store to contain that state again.
     *
     * @throws Error when there is no such state (i.e. when `canUndo()` returns false)
     */
    undo(): void;
    /**
     * Move forward one step in the history of states saved via `pushCurrentState()`, setting the store to contain that state again.
     *
     * @throws Error when there is no such state (i.e. when `canRedo()` returns false)
     */
    redo(): void;
    /**
     * Drops the state from the internal undo stack that would be applied by a call to `.undo()`. This is useful e.g. if making multiple changes in a row that should be collapsed into a single undo state: call this first before pushing the new version.
     *
     * @throws Error when there is no such state (i.e. when `canUndo()` returns false)
     */
    dropCurrentUndoState(): void;
    /**
     * Returns the current undo state that was most recently pushed or applied. Calls to `undo` will apply the state before this in teh stack, and `redo` will apply the state after this.
     */
    get currentUndoState(): UndoStateType;
    /**
     * Returns a view of the internal undo stack, from oldest to newest. Note that this contains states that would be applied by calls to both `.undo()` and `.redo`.
     */
    get undoStack(): UndoStateType[];
    /**
     * Return the information needed to reconstruct the given state. This will be passed to `applyUndoState()` when the store should be reset to this state.
     */
    protected abstract extractUndoState(state: StateType): UndoStateType;
    /**
     * Reset the store to the given state.
     *
     * The `undoOrRedo` and `stateToOverwrite` parameters can be useful e.g. if a scroll position is kept in the undo state. In such a case you want to change the scrolling so the user can see what just changed by undoing/redoing. To do that, set the scoll to what it was in `stateToOverwrite` when undoing, and to what it is in `stateToApply` when redoing.
     */
    protected abstract applyUndoState(stateToApply: UndoStateType, undoOrRedo: UndoOrRedo, stateToOverwrite: UndoStateType): void;
    /**
     * Used to determine whether `.pushCurrentState()` actually does anything. Override this e.g. to prevent pushing a duplicate undo state using something like this:
     *
     * ```ts
     * protected shouldPush(state: UndoStateType) {
     *   return equal(state, this.currentUndoState);
     * }
     * ```
     */
    protected shouldPush(_state: UndoStateType): boolean;
    private dropRedoHistory;
    /**
     * Each time a state is added to the history, this method will be called to determine whether the oldest state should be dropped. Override to implement more complex logic than the simple `maxDepth`.
     */
    protected isOverSize(size: number): boolean;
    private changeState;
    private emitUndoChanges;
    private prepush;
    private actuallyPush;
}
